package ret

import "reflect"

// Panicker is a type which can be used in hel mocks to panic method calls. This
// allows hel to queue a panic instead of a normal method call. Nil values are
// considered normal returns.
type Panicker chan any

// PanickerFieldIdx returns the index of the Panicker field, if any.
func PanickerFieldIdx(mock any) int {
	v := reflect.ValueOf(mock)
	if v.Kind() == reflect.Ptr {
		v = v.Elem()
	}
	if v.Kind() != reflect.Struct {
		return -1
	}
	for i := 0; i < v.NumField(); i++ {
		if _, ok := v.Field(i).Interface().(Panicker); ok {
			return i
		}
	}
	return -1
}
