package pers

import (
	"fmt"
	"reflect"

	"git.sr.ht/~nelsam/hel/vegr/ret"
)

// BlockReturns blocks a mock method from returning until the returned unblock
// function is called. This may be used to pause ConsistentlyReturn or just to
// prevent a method from returning before a test has finished checking the
// application state.
func BlockReturns(t T, mock any, methodName string) (unblock func()) {
	m := reflect.ValueOf(mock)
	if m.Kind() == reflect.Ptr {
		m = m.Elem()
	}
	if m.Kind() != reflect.Struct {
		panic(fmt.Errorf("pers: BlockReturns requires a struct or pointer to a struct as its mock argument; got %T", mock))
	}
	fieldName := methodName + "Output"
	field := m.FieldByName(fieldName)
	if !field.IsValid() {
		panic(fmt.Errorf("pers: struct type %T does not have a field named %v", mock, fieldName))
	}
	if field.Kind() != reflect.Struct {
		panic(fmt.Errorf("pers: field %v of type %T is not a struct type", fieldName, mock))
	}
	for i := 0; i < field.NumField(); i++ {
		subField := field.Field(i)
		blocker, ok := subField.Interface().(ret.Blocker)
		if !ok {
			continue
		}
		<-blocker
		return func() { blocker <- struct{}{} }
	}
	panic(fmt.Errorf("pers: field %v of type %T does not have a blocker field [hint: try regenerating your mocks with the latest hel version]", fieldName, mock))
}
