package pers

import (
	"reflect"

	"git.sr.ht/~nelsam/hel/vegr/ret"
)

// Panic enqueues a panic on a mock method call. This panic will be enqueued
// after any queued returns (using Return).
//
// Panic itself will panic if mock is not a struct with a ret.Panicker field.
func Panic(mock any, withValue any) {
	panicIdx := ret.PanickerFieldIdx(mock)
	if panicIdx == -1 {
		panic("pers: Panic requires a struct with a ret.Panicker field")
	}
	blockerIdx := ret.BlockerFieldIdx(mock)

	v := reflect.ValueOf(mock)
	for i := 0; i < v.NumField(); i++ {
		if i == blockerIdx {
			continue
		}
		f := v.Field(i)
		if f.Kind() != reflect.Chan {
			continue
		}
		if i == panicIdx {
			f.Send(reflect.ValueOf(withValue))
			continue
		}
		v := reflect.Zero(f.Type().Elem())
		f.Send(v)
	}
}
